from __future__ import annotations

from invoke.context import Context
from invoke.tasks import task

from tasks.common import VENV_PREFIX


@task(optional=["clean"])
def build(ctx: Context, clean: bool = True) -> None:
    """Build documentation locally"""
    arguments = " --clean" if clean else ""
    ctx.run(f"{VENV_PREFIX} mkdocs build {arguments}")


@task(default=True)
def serve(ctx: Context) -> None:
    """Run local server"""
    ctx.run(f"{VENV_PREFIX} mkdocs serve")


@task
def deploy(ctx: Context) -> None:
    """Deploy to github page"""
    ctx.run(f"{VENV_PREFIX} mkdocs gh-deploy")



from __future__ import annotations

from invoke.context import Context
from invoke.tasks import task

from tasks.common import VENV_PREFIX


@task
def clean(ctx: Context) -> None:
    """Remove virtual environment"""
    ctx.run("rm -rf .venv", warn=True)


@task
def init(ctx: Context) -> None:
    """Install production dependencies"""
    ctx.run("uv sync --no-dev")


@task
def setup_pre_commit_hook(ctx: Context) -> None:
    """Setup pre-commit hook to automate check before git commit and git push"""
    ctx.run("git init")
    for hook_type in ("pre-commit", "pre-push", "commit-msg"):
        ctx.run(f"{VENV_PREFIX} pre-commit install -t {hook_type}")


@task(optional=["no-pre-commit"])
def init_dev(
    ctx: Context,
    groups_to_install: str = "dev,test,style,security,git",
    no_pre_commit: bool = False,
) -> None:
    """Install development dependencies and setup pre-commit hooks"""
    groups_args = " --group ".join(groups_to_install.split(","))
    groups_args = f"--group {groups_args}"
    ctx.run(f"uv sync {groups_args}")
    if not no_pre_commit:
        setup_pre_commit_hook(ctx)



from __future__ import annotations

from invoke.collection import Collection
from invoke.context import Context
from invoke.tasks import task

from tasks import doc, env, git, secure, style, test
from tasks.build import build_ns
from tasks.common import VENV_PREFIX


@task
def run_pre_commit_hooks(context: Context) -> None:
    """Run pre-commit hooks on all files"""
    context.run(f"{VENV_PREFIX} pre-commit run --all-files --hook-stage pre-push")


ns = Collection()
ns.add_collection(env)  # type: ignore[arg-type]
ns.add_collection(git)  # type: ignore[arg-type]
ns.add_collection(test)  # type: ignore[arg-type]
ns.add_collection(style)  # type: ignore[arg-type]
ns.add_collection(build_ns)
ns.add_collection(secure)  # type: ignore[arg-type]
ns.add_collection(doc)  # type: ignore[arg-type]
ns.add_task(run_pre_commit_hooks)  # type: ignore[arg-type]



from __future__ import annotations

from datetime import datetime

import pandas as pd
import plotly.graph_objects as go


class PelicanDataPlotter:
    def __init__(self, articles_info: list[dict[str, int | str]]):
        self.df = pd.DataFrame(articles_info)
        self.df.timestamp = pd.to_datetime(self.df.timestamp, unit="s")
        self.df.sort_values("timestamp", inplace=True)
        self.df.reset_index(inplace=True, drop=True)
        self.df.index += 1

    def draw_trend_plot(
        self, output_path: str, *, year: int | None = None, groupby_category: bool = False
    ) -> None:
        if year:
            plot_df = self.df[
                (datetime(year, 1, 1) <= self.df.timestamp) & (self.df.timestamp <= datetime(year, 12, 31))
            ]
            plot_df.reset_index(inplace=True, drop=True)
            plot_df.index += 1
        else:
            plot_df = self.df.copy()

        fig = go.Figure()
        if groupby_category:
            for category, cate_df in plot_df.groupby("category"):
                cate_df.reset_index(inplace=True, drop=True)
                cate_df.index += 1
                fig.add_trace(
                    go.Scatter(
                        x=cate_df.timestamp,
                        y=cate_df.index,
                        mode="lines+markers",
                        name=category,
                    )
                )
        else:
            fig.add_trace(
                go.Scatter(
                    x=plot_df.timestamp,
                    y=plot_df.index,
                    mode="lines+markers",
                )
            )
        fig.write_html(output_path)



from __future__ import annotations

import pelican

from pelican_stat import collector


def test_get_pelican_instance():
    pelican_instance = collector.PelicanArticleDataCollector._get_pelican_instance(
        "tests/sample_pelican_project/pelicanconf.py"
    )
    assert isinstance(pelican_instance, pelican.Pelican)


def test_collect_articles():
    data_collector = collector.PelicanArticleDataCollector("tests/sample_pelican_project/pelicanconf.py")
    data_collector.collect_articles()

    assert len(data_collector.articles) == 1

    article = data_collector.articles[0]
    assert article.title == "this is title"
    assert article.category.name == "Testing"
    assert article.slug == "this_is_title"


def test_export(tmp_path, data_regression):
    output_path = tmp_path / "result.json"

    data_collector = collector.PelicanArticleDataCollector("tests/sample_pelican_project/pelicanconf.py")
    data_collector.collect_articles()
    data_collector.export(output_path)

    with open(output_path) as output_file:
        output_data = output_file.read()

    data_regression.check(output_data)



from __future__ import annotations

from invoke.context import Context
from invoke.tasks import task

from tasks.common import VENV_PREFIX


@task(default=True)
def run(ctx: Context, allow_no_tests: bool = False) -> None:
    """Run test cases"""
    result = ctx.run(f"{VENV_PREFIX} pytest", pty=True, warn=True)
    if allow_no_tests and result.exited == 5:
        exit(0)
    exit(result.exited)


@task
def cov(ctx: Context) -> None:
    """Run test coverage check"""
    ctx.run(f"{VENV_PREFIX} pytest --cov=pelican_stat tests/", pty=True)



from __future__ import annotations

import os

import pytest

from pelican_stat import plotter

sample_articles_info = [
    {
        "timestamp": 1611043920.0,
        "category": "Testing",
        "authors": ["Lee-W"],
        "reader": "markdown",
        "status": "published",
        "tags": ["test_tag"],
        "timezone": "Asia/Taipei",
        "title": "this is title",
    }
]


@pytest.mark.parametrize(
    ("year", "groupby_category"), [(None, False), (2020, False), (2020, True), (None, True)]
)
def test_draw_trend_plot(tmpdir, year, groupby_category):
    """Test whether all the valid parameter combination can generate plot

    Note that this test does not check the content of the plot
    """
    with tmpdir.as_cwd():
        output_path = tmpdir / "test_result.html"

        data_plotter = plotter.PelicanDataPlotter(sample_articles_info)
        data_plotter.draw_trend_plot(output_path, year=year, groupby_category=groupby_category)
        assert os.path.exists(output_path)
        os.remove(output_path)



from __future__ import annotations

from invoke.context import Context
from invoke.tasks import task

from tasks.common import VENV_PREFIX


@task
def commit(ctx: Context) -> None:
    """Commit through commitizen"""
    ctx.run(f"{VENV_PREFIX} cz commit", pty=True)


@task
def bump(ctx: Context, changelog: bool = False) -> None:
    """bump version through commitizen"""
    arguments = " --changelog" if changelog else ""
    ctx.run(f"{VENV_PREFIX} cz bump -nr 3 --yes{arguments}", warn=True)



from __future__ import annotations

from invoke.collection import Collection
from invoke.context import Context
from invoke.tasks import task


@task
def clean(ctx: Context) -> None:
    """Remove all the tmp files in .gitignore"""
    ctx.run("git clean -Xdf")


@task
def dist(ctx: Context) -> None:
    """Build distribution"""
    ctx.run("uv build")


@task
def docker(ctx: Context) -> None:
    """Build docker image"""
    ctx.run("uv lock")
    user_name = "lee_w"
    proj_name = "pelican_stat"
    repo_name = f"{user_name}/{proj_name}"
    ctx.run(f"docker build -t {repo_name}:latest .")


build_ns = Collection("build")
build_ns.add_task(clean)  # type: ignore[arg-type]
build_ns.add_task(dist)  # type: ignore[arg-type]
build_ns.add_task(docker)  # type: ignore[arg-type]



#!/usr/bin/env python
from __future__ import annotations

AUTHOR = "test"
SITENAME = "testtest"
SITEURL = ""

PATH = "content"

TIMEZONE = "Asia/Taipei"

DEFAULT_LANG = "zh"

# Feed generation is usually not desired when developing
FEED_ALL_ATOM = None
CATEGORY_FEED_ATOM = None
TRANSLATION_FEED_ATOM = None
AUTHOR_FEED_ATOM = None
AUTHOR_FEED_RSS = None

# Blogroll
LINKS = (
    ("Pelican", "https://getpelican.com/"),
    ("Python.org", "https://www.python.org/"),
    ("Jinja2", "https://palletsprojects.com/p/jinja/"),
    ("You can modify those links in your config file", "#"),
)

# Social widget
SOCIAL = (
    ("You can add links in your config file", "#"),
    ("Another social link", "#"),
)

DEFAULT_PAGINATION = 10

# Uncomment following line if you want document-relative URLs when developing
# RELATIVE_URLS = True



from __future__ import annotations

from click.testing import CliRunner

from pelican_stat.__main__ import main
from pelican_stat.collector import PelicanArticleDataCollector
from pelican_stat.plotter import PelicanDataPlotter


class TestCollectCommand:
    def test_normal_case(self, mocker):
        mocked_export_method = mocker.patch.object(PelicanArticleDataCollector, "export")

        runner = CliRunner()
        result = runner.invoke(main, ["collect", "tests/sample_pelican_project/pelicanconf.py"])
        assert result.exit_code == 0
        mocked_export_method.assert_called_once()

    def test_pelican_conf_not_found(self):
        runner = CliRunner()
        result = runner.invoke(main, ["collect"])
        assert result.exit_code == 1


class TestPlotCommand:
    def test_normal_case(self, mocker):
        mocked_draw_method = mocker.patch.object(PelicanDataPlotter, "draw_trend_plot")

        runner = CliRunner()
        result = runner.invoke(
            main,
            [
                "plot",
                "--pelican-conf-path",
                "tests/sample_pelican_project/pelicanconf.py",
            ],
        )
        assert result.exit_code == 0
        mocked_draw_method.assert_called_once()



from __future__ import annotations

import json
import os
import sys

import click

from pelican_stat.collector import PelicanArticleDataCollector
from pelican_stat.plotter import PelicanDataPlotter


@click.group()
def main() -> None:
    pass


@main.command()
@click.argument(
    "pelican_conf_path",
    required=True,
    default="pelicanconf.py",
)
@click.argument(
    "output_path",
    required=True,
    default="article_metadata.json",
)
def collect(pelican_conf_path: str, output_path: str) -> None:
    """Collect data from pelican project and export article metadata"""
    if not os.path.exists(pelican_conf_path):
        print(f"Configuration file {pelican_conf_path} does not exists", file=sys.stderr)
        sys.exit(1)

    data_collector = PelicanArticleDataCollector(pelican_conf_path)
    data_collector.export(output_path)


@main.command()
@click.option(
    "--pelican-conf-path",
    help=(
        "path to pelican site configuration file (e.g., pelicanconf.py). "
        "note that either --pelican-conf-path or --articles-metadata-path "
        "needs to be provided"
    ),
)
@click.option(
    "--articles-metadata-path",
    help=(
        "path to collected articles metadata. "
        "note that either --pelican-conf-path or --articles-metadata-path "
        "needs to be provided"
    ),
)
@click.option(
    "--output-path",
    required=True,
    default="trend_plot.html",
    help="path for the output plot",
)
@click.option("--year", required=False, help="plot only the data for certain year", type=int)
@click.option(
    "--groupby-category",
    required=False,
    help="group data by category",
    is_flag=True,
)
def plot(
    pelican_conf_path: str = "",
    articles_metadata_path: str = "",
    output_path: str = "trend_plot.html",
    year: int | None = None,
    groupby_category: bool = False,
) -> None:
    """Draw trend plot based on the frequency of new posts"""
    if not pelican_conf_path and not articles_metadata_path:
        print(
            "Neither pelican_conf_path nor articles_metadata_path is provided",
            file=sys.stderr,
        )
        sys.exit(1)

    if (pelican_conf_path and not os.path.exists(pelican_conf_path)) or (
        articles_metadata_path and not os.path.exists(articles_metadata_path)
    ):
        print(
            f"{pelican_conf_path or articles_metadata_path} does not exist",
            file=sys.stderr,
        )
        sys.exit(1)

    if pelican_conf_path:
        data_collector = PelicanArticleDataCollector(pelican_conf_path)
        articles_info = data_collector.extract_articles_info()
    elif articles_metadata_path:
        with open(articles_metadata_path) as metadata_file:
            articles_info = json.load(metadata_file)
    else:
        raise ValueError("Either pelican conf path or metadata path should be provided.")

    data_plotter = PelicanDataPlotter(articles_info)
    data_plotter.draw_trend_plot(output_path, year=year, groupby_category=groupby_category)


if __name__ == "__main__":
    main()



from __future__ import annotations

from invoke.context import Context
from invoke.tasks import task

from tasks.common import VENV_PREFIX


@task
def check_package(ctx: Context) -> None:
    """Check package security"""
    ctx.run(f"{VENV_PREFIX} pip-audit", warn=True)


@task
def bandit(ctx: Context) -> None:
    """Check common software vulnerabilities (Use it as reference only)"""
    ctx.run(f"{VENV_PREFIX} bandit -r -iii -lll --ini .bandit", pty=True)


@task(pre=[check_package, bandit], default=True)
def run(ctx: Context) -> None:
    """Check security check through pip-audit and bandit"""
    pass



from __future__ import annotations

import json
from pathlib import Path

import pelican


class PelicanArticleDataCollector:
    def __init__(self, pelican_conf_path: str) -> None:
        self.pelican_instance = self._get_pelican_instance(pelican_conf_path)
        self.articles: list[pelican.contents.Article] = []
        self.collect_articles()

    @staticmethod
    def _get_pelican_instance(conf_path: str) -> pelican.Pelican:
        settings = pelican.read_settings(conf_path)
        settings["PLUGINS"] = []
        settings["MARKDOWN"] = {}

        cls = settings["PELICAN_CLASS"]
        if isinstance(cls, str):
            module_name, cls_name = cls.rsplit(".", 1)
            module = __import__(module_name)
            cls = getattr(module, cls_name)

        return cls(settings)

    def collect_articles(self) -> None:
        article_generator_cls = self.pelican_instance._get_generator_classes()[0]

        context = self.pelican_instance.settings.copy()
        context["generated_content"] = {}
        context["static_links"] = set()
        context["static_content"] = {}
        context["localsiteurl"] = self.pelican_instance.settings["SITEURL"]

        article_generator = article_generator_cls(
            context=context,
            settings=self.pelican_instance.settings,
            path=self.pelican_instance.path,
            theme=self.pelican_instance.theme,
            output_path=self.pelican_instance.output_path,
        )
        article_generator.generate_context()
        self.articles = article_generator.articles

    def extract_articles_info(self) -> list[dict[str, str | int]]:
        articles_info: list[dict[str, str | int]] = [
            {
                "timestamp": article.date.timestamp(),
                "category": article.category.name,
                "authors": [author.name for author in article.authors],  # type: ignore
                "reader": article.reader,
                "status": article.status,
                "tags": [tag.name for tag in article.tags],  # type: ignore
                "timezone": str(article.timezone),
                "title": article.title,
            }
            for article in self.articles
        ]
        return articles_info

    def export(self, output_path: Path | str) -> None:
        articles_info = self.extract_articles_info()
        with open(output_path, "w") as f:
            json.dump(articles_info, f, ensure_ascii=False, indent=4)



from __future__ import annotations

from invoke.context import Context
from invoke.tasks import task

from tasks.common import COMMON_TARGETS_AS_STR, VENV_PREFIX


@task
def ruff(ctx: Context, fix: bool = True) -> None:
    """Check style through ruff"""
    arguments = " --fix" if fix else ""
    ctx.run(f"{VENV_PREFIX} ruff check {COMMON_TARGETS_AS_STR} {arguments}")


@task
def mypy(ctx: Context) -> None:
    """Check style through mypy"""
    ctx.run(f"{VENV_PREFIX} mypy")


@task
def commit_check(ctx: Context, remote: str = "origin") -> None:
    """Check commit message through commitizen"""
    ctx.run(
        f"{VENV_PREFIX} cz -nr 3 check --rev-range {remote}/main..",
        warn=True,
    )


@task(pre=[ruff, mypy, commit_check], default=True)
def run(ctx: Context) -> None:
    """Check style through linter"""
    pass


@task
def ruff_format(ctx: Context) -> None:
    """Format Python code through ruff"""
    ctx.run(f"{VENV_PREFIX} ruff format {COMMON_TARGETS_AS_STR}")


@task(pre=[ruff_format])
def format(ctx: Context) -> None:
    """Reformat python files through ruff"""
    pass
